@@ -5,7 +5,7 @@
 NULL
 
 globalVariables(c("from", "i", "id", "label", "to", "x", "shap",
-                  "feature", "value", "color", "Var2", "Var3"))
+                  "feature", "value", "color", "Var2", "Var3", "S"))
 
 .onLoad <- function(libname, pkgname) {
   op <- options()
